#ifndef serial_HH
#define serial_HH

#include <string>
#include <sys/ioctl.h>

using std::string;

enum ComSpeed
{
	CS200,
	CS300,
	CS600,
	CS1200,
	CS2400,
	CS4800,
	CS9600,
	CS19200,
	CS38400,
	CS57600,
	CS115200,
	CS230400,
};

enum ComParity
{
	CPODD,
	CPEVEN,
	CPNONE,
};

class ComPort {
public:
   ComPort     (ComPort &);
   ComPort		(int number, ComSpeed speed, ComParity parity, int data_bits, int stop_bits);
   ComPort		(	const char *port,
						ComSpeed speed,
						ComParity parity,
						int data_bits,
						int stop_bits);
	~ComPort		();

	bool IsOpen	() {return (bool)(fd>0); }

	void SendBreak ();
    bool ToggleDTR ();
    bool ToggleRTS ();
    int  GetModemFlags ();

    bool StateDCD (int flags) { return flags&TIOCM_CAR; }
    bool StateDSR (int flags) { return flags&TIOCM_DSR; }
    bool StateRNG (int flags) { return flags&TIOCM_RNG; }
    bool StateCTS (int flags) { return flags&TIOCM_CTS; }

	int Write	(const char *buff, int len);
	int Read	(char *buff, int max_size);

	int GetFd   () { return fd; }

	int SpeedToInt (ComSpeed speed)
	{
		switch (speed)
		{
			case CS200:    return 200;
			case CS300:    return 300;
			case CS600:    return 600;
			case CS1200:   return 1200;
			case CS2400:   return 2400;
			case CS4800:   return 4800;
			default:
			case CS9600:   return 9600;
			case CS19200:  return 19200;
			case CS38400:  return 38400;
			case CS57600:  return 57600;
			case CS115200: return 115200;
			case CS230400: return 230400;
		}
		return 0;
	}

   int GetSpeedValue () { return speed_val; }

	string GetSpeedAsString (ComSpeed s);
	string GetParityAsString (ComParity p);

private:
	void InitPort ( 	const char *port,
							ComSpeed speed,
							ComParity parity,
							int data_bits,
							int stop_bits );

	int TranslateSpeed	(ComSpeed s);
	int TranslateParity	(ComParity p);

private:
	int fd;
	int speed_val;
};

#endif // serial_HH
