/*
 * DlgPref, preferences dialog
 *
 * (C) Copyright 2011 Angelo Dureghello <angelo70@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include "dlgpref.hh"

#include <wx/propgrid/propgrid.h>
#include <wx/propgrid/property.h>
#include <wx/propgrid/advprops.h>
#include <wx/xrc/xmlres.h>
#include <wx/config.h>

BEGIN_EVENT_TABLE( DlgPref, wxDialog )
   EVT_BUTTON     ( wxID_OK, DlgPref::OnOk )
END_EVENT_TABLE()

DlgPref::DlgPref ()
{
}

const char *baud[] = {
   "200",
	"300",
	"600",
	"1200",
	"2400",
	"4800",
	"9600",
	"19200",
	"38400",
	"57600",
	"115200",
	"230400",
	0
};

const char *parity[] = {
   "Odd",
	"Even",
	"None",
	0
};

void DlgPref::Init ()
{
   wxPanel *p = (wxPanel*) XRCCTRL(*this, "paGrid", wxPanel);

   pg = new wxPropertyGrid(p, -1);

   SetSize(400,400);

   pg->SetSize(p->GetSize());

   pg->Append(new wxPropertyCategory("Terminal"));

   pg->Append(new wxColourProperty("Backgorund color", "bgcol"));
   pg->Append(new wxColourProperty("Foreground color", "fgcol"));
   pg->Append(new wxFontProperty("Font", "font"));
   pg->Append(new wxIntProperty("Buffer level", "bufle"));
   pg->Append(new wxBoolProperty("Translate NL to CR on input",
                                    "nl_to_cr", true));

   pg->Append(new wxPropertyCategory("Communication"));

   pg->Append(new wxStringProperty("Serial port", "devse"));
   pg->Append(new wxEnumProperty("Baud rate", "baudr", baud));
   pg->Append(new wxEnumProperty("Parity", "parit", parity));
   pg->Append(new wxIntProperty("Data bits", "datab"));
   pg->Append(new wxIntProperty("Stop bits", "stopb"));

   SetupValues();

   CenterOnParent();
}

void DlgPref::SetupValues ()
{
    wxConfig::Get()->SetPath("/Terminal");

    wxVariant v;

    wxColor b(wxConfig::Get()->Read("bgcol"));
    v << b;
    pg->SetPropertyValue("bgcol", v);

    wxColor f(wxConfig::Get()->Read("fgcol"));
    v << f;
    pg->SetPropertyValue("fgcol", v);
    pg->SetPropertyValue("font", wxConfig::Get()->Read("font"));

    pg->SetPropertyValue("bufle", wxConfig::Get()->Read("bufle"));
    pg->SetPropertyAttribute ("bufle", wxPG_ATTR_MAX, 100000);
    pg->SetPropertyAttribute ("bufle", wxPG_ATTR_MIN, 1000);
    pg->SetPropertyValue ("nl_to_cr", wxConfig::Get()->ReadBool("nl_to_cr",1));
    pg->SetPropertyAttribute ("nl_to_cr", "UseCheckbox", 1);

    wxConfig::Get()->SetPath("/Communication");

    pg->SetPropertyValue("devse", wxConfig::Get()->Read("devse"));
    pg->SetPropertyValue("baudr", wxConfig::Get()->Read("baudr", 6));
    pg->SetPropertyValue("parit", wxConfig::Get()->Read("parit", 2));
    pg->SetPropertyValue("datab", wxConfig::Get()->Read("datab"));
    pg->SetPropertyAttribute ("datab", wxPG_ATTR_MAX, 8);
    pg->SetPropertyAttribute ("datab", wxPG_ATTR_MIN, 5);
    pg->SetPropertyValue("stopb", wxConfig::Get()->Read("stopb"));
    pg->SetPropertyAttribute ("stopb", wxPG_ATTR_MAX, 2);
    pg->SetPropertyAttribute ("stopb", wxPG_ATTR_MIN, 1);
}

void DlgPref::OnOk (wxCommandEvent &)
{
    wxVariant v;
    wxColour c;

    if (pg->ClearSelection(true) == false) return;

    wxConfig::Get()->SetPath("/Terminal");

    v = pg->GetPropertyValue("bgcol");
    c << v;
    wxConfig::Get()->Write("bgcol",c.GetAsString());

    v = pg->GetPropertyValue("fgcol");
    c << v;
    wxConfig::Get()->Write("fgcol",c.GetAsString());

    wxString font=pg->GetPropertyValueAsString("font");

    wxConfig::Get()->Write("font",  font);

    wxConfig::Get()->Write("bufle", pg->GetPropertyValueAsLong("bufle"));
    wxConfig::Get()->Write("nl_to_cr", pg->GetPropertyValueAsBool("nl_to_cr"));

    wxConfig::Get()->SetPath("/Communication");

    wxConfig::Get()->Write("devse", pg->GetPropertyValueAsString("devse"));
    wxConfig::Get()->Write("baudr", pg->GetPropertyValueAsLong("baudr"));
    wxConfig::Get()->Write("parit", pg->GetPropertyValueAsLong("parit"));
    wxConfig::Get()->Write("datab", pg->GetPropertyValueAsLong("datab"));
    wxConfig::Get()->Write("stopb", pg->GetPropertyValueAsLong("stopb"));

    wxConfig::Get()->Flush();

    EndModal (wxID_OK);
}
