/*
 * FrmMain class
 *
 * Main UI implementatiuon
 *
 * (C) Copyright 2011 Angelo Dureghello <angelo70@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <wx/xrc/xmlres.h>
#include <wx/toolbar.h>
#include <wx/artprov.h>
#include <wx/statusbr.h>
#include <wx/menu.h>
#include <wx/button.h>
#include <wx/config.h>
#include <wx/msgdlg.h>
#include <wx/stattext.h>
#include <wx/sizer.h>

#include <sstream>

#include "frmmain.hh"
#include "xymodem.hh"
#include "raw.hh"
#include "serial.hh"
#include "dlgsend.hh"
#include "dlgpref.hh"
#include "vteterm.hh"
#include "pixmaps.hh"
#include "version.h"

using std::stringstream;

enum ids {
   id_open=101,
   id_exit,
   id_clear,
   id_pref,
   id_sendbreak,
   id_toggle_dtr,
   id_toggle_rts,
   id_about,
};

static const int max_signals=6;

static int st_widths[max_signals+2] =
{0 /*to be set later*/, 32, 32, 32, 32, 32, 32, 32};

static int st_styles[max_signals+2] =
{
	wxSB_FLAT,
	wxSB_NORMAL,
	wxSB_NORMAL,
	wxSB_FLAT,
	wxSB_FLAT,
	wxSB_FLAT,
	wxSB_FLAT,
	wxSB_FLAT
};

/* first are the DTE enabled signals */
static const char signals[max_signals][4] =
{"DTR","RTS","DCD","DSR","RNG","CTS"};

BEGIN_EVENT_TABLE( FrmMain, wxFrame )
	EVT_SIZE ( FrmMain::OnSize )
	//EVT_IDLE ( FrmMain::OnIdle )
	EVT_MENU ( id_open, FrmMain::OnSend )
	EVT_MENU ( id_exit, FrmMain::OnQuit )
	EVT_MENU ( id_clear, FrmMain::OnClear )
	EVT_MENU ( id_pref, FrmMain::OnPref )
	EVT_MENU ( id_sendbreak, FrmMain::OnSendBreak )
	EVT_MENU ( id_toggle_dtr, FrmMain::OnToggleDTR )
	EVT_MENU ( id_toggle_rts, FrmMain::OnToggleRTS )
	EVT_MENU ( id_about, FrmMain::OnAbout )
	EVT_CLOSE   ( FrmMain::OnClose )
   EVT_COMMAND ( wxID_ANY, wxEVT_THREAD, FrmMain::OnThread )
END_EVENT_TABLE()


static const int MAX_BUFF = (128*1024);

FrmMain::FrmMain() : t(0), p(0), tim(this, wxID_ANY)
{
	buff.resize(MAX_BUFF);

	rx_enabled = true;

	ctrl = false;

	wxXmlResource::Get()->LoadFrame (this, 0, wxT("frmMain"));

	sb = GetStatusBar();

	wxMenu* me;
	wxMenuItem *it;

	me = GetMenuBar()->GetMenu(0);
	it = new wxMenuItem (me, id_open, "Upload file",
						"Upload a file into remote device", wxITEM_NORMAL);
	it->SetBitmap(xpm_16_open);
	me->Append(it);

	it = new wxMenuItem (me, id_exit, "Exit",
						"Terminate the application", wxITEM_NORMAL);
	it->SetBitmap(xpm_16_exit);
	me->AppendSeparator();
	me->Append(it);

	me = GetMenuBar()->GetMenu(1);
	it = new wxMenuItem (me, id_clear, "Clear screen",
						"Clear the terminal screen", wxITEM_NORMAL);
	it->SetBitmap(xpm_16_clear);
	me->Append(it);

	it = new wxMenuItem (me, id_pref, "Preferences",
						"Application setup", wxITEM_NORMAL);

	me->AppendSeparator();
	it->SetBitmap(xpm_16_pref);
	me->Append(it);

	me = GetMenuBar()->GetMenu(2);
	it = new wxMenuItem (me, id_sendbreak, "Send break\tCtrl+B",
						"Send a break", wxITEM_NORMAL);
	me->Append(it);
	it = new wxMenuItem (me, id_toggle_dtr, "Toggle DTR",
						"Toggle DTR signal", wxITEM_NORMAL);
	me->Append(it);
	it = new wxMenuItem (me, id_toggle_rts, "Toggle RTS",
						"Toggle RTS signal", wxITEM_NORMAL);
	me->Append(it);

	me = GetMenuBar()->GetMenu(3);
	it = new wxMenuItem (me, id_about, "About ...",
						"About microterm", wxITEM_NORMAL);
	me->Append(it);

	wxToolBar *tb = CreateToolBar(wxNO_BORDER | wxTB_VERTICAL);

	/* in wxglade i'm enable toolbar in frame-properties dialog */
	tb->AddTool (id_open,  "Upload", xpm_22_open,  "Upload file");
	tb->AddTool (id_clear, "Clear",  xpm_22_clear, "Clear the terminal");
	tb->AddTool (id_pref,  "Preferences",  xpm_22_pref, "Preferences");

	wxSizer *sizer = new wxBoxSizer(wxVERTICAL);
        SetSizer(sizer);

        t = new VTETerm (this, 10100, "", wxDefaultPosition, wxDefaultSize);

        sizer->Add(t, 1, wxEXPAND | wxALL, 0);

        sizer->Layout();
        Layout();

	Setup();
}

/*
 * first field is status, 6 fields are signals, last is
 * just a blank space field.
 */
void FrmMain::SetupStatusBar(int client_width)
{
	st_widths[0]=client_width-(32*max_signals)-32;

	sb->SetFieldsCount	(max_signals+2);
	sb->SetStatusWidths	(max_signals+2,st_widths);
	sb->SetStatusStyles	(max_signals+2,st_styles);

	for (int i=1;i<max_signals+1;++i)
	{
		sb->SetStatusText (signals[i-1], i);
	}
}

void FrmMain::Setup ()
{
	wxConfig::Get()->SetPath("/Terminal");

	wxColor f(wxConfig::Get()->Read ("fgcol"));
	wxColor b(wxConfig::Get()->Read ("bgcol"));

	t->SetColours (f.GetAsString(wxC2S_HTML_SYNTAX).ToAscii(),
		  b.GetAsString(wxC2S_HTML_SYNTAX).ToAscii());

	t->SetFont (wxConfig::Get()->Read ("font"));

	/* calculate minimal 80x24 */
	int x,y,w,h;

	w = wxConfig::Get()->Read ("wwidth", 640);
	h = wxConfig::Get()->Read ("wheight", 480);
	x = wxConfig::Get()->Read ("wx", 10);
	y = wxConfig::Get()->Read ("wy", 10);

	if (w==640 && h==480) { // defaults
		w=80*t->GetCharWidth();
		h=24*t->GetCharHeight();

		if (w&&h)
			SetSize(w,h);
		else
			SetSize (640,480);
	}
	else SetSize (w,h);

	SetPosition (wxPoint(x, y));

	t->SetBufferLines (wxConfig::Get()->Read ("bufle", 5000));

	wxConfig::Get()->SetPath("/Communication");

	if (p) delete (p);

	wxString port = wxConfig::Get()->Read ("devse");
	long baudrate = wxConfig::Get()->Read ("baudr", 6);
	long parity   = wxConfig::Get()->Read ("parit", 2);
	long databits = wxConfig::Get()->Read ("datab", 8);
	long stopbits = wxConfig::Get()->Read ("stopb", 1);

	p = new ComPort( port.ToAscii(),
			(ComSpeed) baudrate,
			(ComParity) parity,
			databits,
			stopbits );

	t->clear();

	if (!p->IsOpen())
	{
		rx_enabled = false;
		t->Enable (false);

		wxString Msg = wxString("Error opening port ")+port;

		sb->SetStatusText(Msg);
		t->write (Msg.ToAscii(), Msg.Len());
	}
	else
	{
		tim.Start(20);
		Connect(tim.GetId(), wxEVT_TIMER, wxTimerEventHandler(FrmMain::OnTimer), NULL, this);

		/* are they connected ? There is no wy to know it, so disconnect them */
		t->Disconnect(wxEVT_KEY_DOWN, wxKeyEventHandler(FrmMain::OnKeyDown));
		t->Disconnect(wxEVT_KEY_UP, wxKeyEventHandler(FrmMain::OnKeyUp));

		t->Connect(wxEVT_KEY_DOWN, wxKeyEventHandler(FrmMain::OnKeyDown), NULL, this);
		t->Connect(wxEVT_KEY_UP,   wxKeyEventHandler(FrmMain::OnKeyUp), NULL, this);

		stringstream ss;

		ss << p->GetSpeedAsString((ComSpeed)baudrate) << ","
		<< p->GetParityAsString((ComParity)parity) << ","
		<< databits << "," << stopbits;

		sb->SetStatusText(wxString("Ready : ")+port+","+ss.str());
	}
}

void FrmMain::OnSendBreak (wxCommandEvent &)
{
	p->SendBreak();
}

void FrmMain::OnToggleDTR (wxCommandEvent &)
{
	st_styles[1] = (p->ToggleDTR()) ? wxSB_NORMAL : wxSB_FLAT;
	sb->SetStatusStyles	(max_signals+2,st_styles);
}

void FrmMain::OnToggleRTS (wxCommandEvent &)
{
	st_styles[2] = (p->ToggleRTS()) ? wxSB_NORMAL : wxSB_FLAT;
	sb->SetStatusStyles	(max_signals+2,st_styles);
}

void FrmMain::OnSize (wxSizeEvent &e)
{
	if (t) {
		wxSize csz=e.GetSize();

		SetupStatusBar (csz.GetWidth());
		Layout ();
	}
}

void FrmMain::OnIdle (wxIdleEvent &/*e*/)
{
}

void FrmMain::OnKeyDown (wxKeyEvent &e)
{
	static unsigned char c;

	c = e.GetRawKeyCode()&0xff;

	switch(c)
	{
		case 0xe3:
			ctrl=true;
			// want to fall thorugh !
		case 0xe1:  // shift left
		case 0xe2:  // shift right
			return;
		default:
			break;
	}
	if (ctrl)
	{
		if (c=='c') c=0x03;
	}
	if (p->Write ((char*)&c, 1) == -1)
		wxMessageBox("Error writing on serial port, check device type and permissions.",
			   "ERROR");

	e.Skip(true);
}

void FrmMain::OnKeyUp (wxKeyEvent &e)
{
	static int i;

	i = e.GetRawKeyCode()&0xff;

	switch (i) {
		case 0xe3: ctrl=false; break;
	}
}

void FrmMain::OnSend (wxCommandEvent &)
{
   DlgSend ds;

	wxXmlResource::Get()->LoadDialog(&ds, this, _T("dlgSend"));

	ds.Init	();

	if (ds.ShowModal() != wxID_CANCEL)
	{
		string file=ds.GetFile();

		if (file=="")
		{
			wxMessageBox("No file selected.", "ERROR");
			return;
		}

		/* disable sending and receiving from FrmMain */
		t->Enable (false);
		rx_enabled = false;

		Operator *o=0;

		switch(ds.GetProto())
		{
			case p_raw:
			{
				new RawSender (*p, this, file);
				return;
			}
			break;
			case p_xmodem:
				o = new XModem(*p, this);
			break;
			case p_ymodem:
				o = new YModem(*p, this);
			break;
		}
		o->SessionStartup (file);
		/* go */
		o->StartTransfert ();
	}
}

void FrmMain::OnClose (wxCloseEvent &)
{
	wxConfigBase *c = wxConfig::Get();

	if (!c->Exists("Terminal"))
	{
		int x,y,w,h;

		GetSize 	(&w, &h);
		GetPosition (&x, &y);

		c->SetPath ("/Terminal");
		c->Write ("wx", x);
		c->Write ("wy", y);
		c->Write ("wwidth", w);
		c->Write ("wheight", h);

		c->Flush ();
	}
	Destroy();
}

void FrmMain::OnQuit (wxCommandEvent &)
{
	Close();
}

void FrmMain::OnClear (wxCommandEvent &)
{
	t->clear();
}

void FrmMain::OnPref  (wxCommandEvent &)
{
	DlgPref dp;

	wxXmlResource::Get()->LoadDialog(&dp, this, _T("dlgPref"));

	dp.Init			();

	if (dp.ShowModal() != wxID_CANCEL)
	{
		/* update values here */
		Setup();
	}
}

void FrmMain::OnAbout (wxCommandEvent &)
{
	wxDialog da;

	wxXmlResource::Get()->LoadDialog(&da, this, _T("dlgAbout"));

	wxStaticText *stVersion = (wxStaticText*) XRCCTRL(*this, "stVersion", wxStaticText);

	stVersion->SetLabel (wxString ("v.")+version);

	da.SetSize (300,240);
	da.CenterOnParent ();

	da.ShowModal ();
}

void FrmMain::OnThread (wxCommandEvent& event)
{
	switch (event.GetId())
	{
		case evt_log:
			t->write(event.GetString(),event.GetString().Len());
		break;
		case evt_del:
			t->write("\x1b[1K\r", 5);
		break;
		case evt_end:
			t->Enable (true);
			rx_enabled = true;
		break;
		default:
		break;
	}
}

void FrmMain::UpdateDCEFlags (int flags)
{
	st_styles[3] = (p->StateDCD(flags)) ? wxSB_NORMAL : wxSB_FLAT;
	st_styles[4] = (p->StateDSR(flags)) ? wxSB_NORMAL : wxSB_FLAT;
	st_styles[5] = (p->StateRNG(flags)) ? wxSB_NORMAL : wxSB_FLAT;
	st_styles[6] = (p->StateCTS(flags)) ? wxSB_NORMAL : wxSB_FLAT;
	sb->SetStatusStyles	(max_signals+2,st_styles);
}

void FrmMain::OnTimer (wxTimerEvent &/*event*/)
{
	static int len;
	static int modem_flags_check=0, modem_flags=-1;

	if (rx_enabled)
	{
		while ((len = p->Read(&buff[0], MAX_BUFF))>0)
		{
			buff[len]=0;

			t->write(buff, len);
		}
		if (++modem_flags_check==10)
		{
			int new_flags = p->GetModemFlags();

			if (modem_flags!=new_flags) modem_flags=new_flags;
			UpdateDCEFlags(modem_flags);

			modem_flags_check=0;
		}
	}
}

