/**
 *
 * microterm :)
 *
 * (C) Copyright 2011 Angelo Dureghello <angelo70@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <wx/app.h>
#include <wx/stdpaths.h>
#include <wx/config.h>
#include <wx/xrc/xmlres.h>
#include <wx/msgdlg.h>

#include "frmmain.hh"

class MicroTerm : public wxApp
{
public:
   virtual bool OnInit();

private:
   void CheckConfig();
};

IMPLEMENT_APP(MicroTerm)

void MicroTerm::CheckConfig()
{
	wxConfig *c = new wxConfig("microterm");

	if (!c->Exists("Terminal"))
	{
		/* setup defaults now */
		c->DeleteAll();

		c->SetPath ("/Terminal");
		c->Write ("fgcol", "rgb(0, 0, 0)");
		c->Write ("bgcol", "rgb(255, 255, 255)");
		c->Write ("font", "10; Monospace; Normal; Normal; Not Underlined; Teletype");
		c->Write ("bufle", 5000);
		c->Write ("nl_to_cr", true);
		c->Write ("wwidth", 640);
		c->Write ("wheight", 480);

		c->SetPath ("/Communication");
		c->Write ("devse", "/dev/ttyS0");
		c->Write ("baudr", 6);
		c->Write ("parit", 2);
		c->Write ("datab", 8);
		c->Write ("stopb", 1);

		c->Flush ();
	}
}

static const int MAX_PATHS = 8;
static const int MAX_PATH_LEN = 128;
static const char paths[MAX_PATHS][MAX_PATH_LEN] = {
	{"/usr/share/microterm/"},
	{"/usr/local/share/microterm/"},
	{0}
};

bool MicroTerm::OnInit()
{
	wxStandardPaths sp;

	wxString path = sp.GetExecutablePath();

#ifndef WIN32
	path=path.substr(0,path.rfind('/'));
#else
	path=path.substr(0,path.rfind('\\'));
#endif

	CheckConfig();

	string resource,icon;
	int i;
	for (i=0;i<MAX_PATHS;++i)
	{
		resource=paths[i];
		if (wxFileExists(resource+"microterm.xrc"))
			break;
	}

	if (i==MAX_PATHS)
	{
		// last chance, from current path, res subfolder
		resource=path+"/res/";

		if (!wxFileExists(resource+"microterm.xrc"))
		{
			string msg;

			msg+="Resource file missing\n"
				 "check for microterm.xrc to exist, as\n"
				 "./res/microterm.xrc\n";

			for (i=0;(i<MAX_PATHS)&&(paths[i][0]!=0);++i)
			{
				msg+=paths[i];
				msg+="\n";
			}

			wxMessageDialog d (0, msg, "ERROR", wxOK | wxICON_ERROR);
			d.ShowModal();

			return false;
		}
	}

	wxXmlResource::Get()->InitAllHandlers ();
	wxXmlResource::Get()->Load(resource+"microterm.xrc");

	FrmMain *f = new FrmMain;

	wxInitAllImageHandlers();
	f->SetIcon(wxIcon(resource+"/microterm.png", wxBITMAP_TYPE_PNG));

	f->Show(TRUE);

	SetTopWindow(f);

	return TRUE;
}
