/*
 * vteterm c++ wrapper
 *
 * (C) Copyright 2011 Angelo Dureghello <angelo70@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include "vteterm.hh"

#include <vte/vte.h>
#include <wx/window.h>
#include <gdk/gdk.h>


VTETerm::VTETerm () {}

VTETerm::VTETerm (wxWindow *parent,
                           wxWindowID id,
                           const wxString &label,
                           const wxPoint &pos,
                           const wxSize &size,
                           long style,
                           const wxString &name)
{
   Create (parent, id, label, pos, size, style, name);
}

bool VTETerm::Create (wxWindow *parent,
                           wxWindowID id,
                           const wxString &,
                           const wxPoint &pos,
                           const wxSize &size,
                           long style,
                           const wxString &name)
{

    if (!PreCreation( parent, pos, size) ||
    !CreateBase( parent, id, pos, size, style, wxDefaultValidator, name ))
    {
        wxFAIL_MSG( wxT("wxWebKitCtrl creation failed") );
        return FALSE;
    }

    m_widget = GTK_WIDGET(vte_terminal_new());

    m_parent->DoAddChild( this );

    PostCreation(size);

    vte_terminal_set_emulation ((VteTerminal*)m_widget, "xterm");
    vte_terminal_set_scroll_on_keystroke ((VteTerminal*)m_widget, 1);

    return true;
}

void VTETerm::SetBufferLines (int lines)
{
   vte_terminal_set_scrollback_lines ((VteTerminal*)m_widget, lines);
}

void VTETerm::DoSetSize(int x, int y,
                           int width, int height,
                           int sizeFlags )
{
   wxControl::DoSetSize( x, y, width, height, sizeFlags );
}

wxSize VTETerm::DoGetBestSize() const
{
   // Do not return any arbitrary default value...
   wxASSERT_MSG( m_widget, wxT("wxWebKitCtrl::DoGetBestSize called before creation") );

   GtkRequisition req;
   req.width = -1;
   req.height = -1;
   (* GTK_WIDGET_CLASS( GTK_OBJECT_GET_CLASS(m_widget) )->size_request )
      (m_widget, &req );

   // Adding 1 to width to workaround GTK sometimes wrapping the text needlessly
   return wxSize (req.width+1, req.height);
}

/*
 * Note: len allows binary codes to be processed from the terminal.
 */
void VTETerm::write (const string &text, int len)
{
   vte_terminal_feed  ((VteTerminal*)m_widget, text.c_str(), len);
}

void VTETerm::clear ()
{
   vte_terminal_reset((VteTerminal*)m_widget, 1, 1);
}

void VTETerm::SetColours (const char *fg, const char *bg)
{
   GdkColor *f,*b;

   f = new GdkColor;
   b = new GdkColor;

   gdk_color_parse (bg, b);
   gdk_color_parse (fg, f);

   vte_terminal_set_colors  ((VteTerminal*)m_widget, f, b, 0, 0);

   gtk_widget_realize (m_widget);
}

string VTETerm::WxFontStringToPango (const string &f)
{

   /* pango string requested */
   /* "serif,monospace bold italic condensed 16" */
   /*"[FAMILY-LIST] [STYLE-OPTIONS] [SIZE]

   "12; Sans; Normal; Normal; Not Underlined; Swiss"); */

   string size;
   string family_name;
   string style = "";

   unsigned int i,l;
   if ((i=f.find(';'))!=string::npos)
   {
      size=f.substr(0,i);

      if ((l=f.find(';',i+1))!=string::npos)
      {
         family_name=f.substr(i+2,l-i-2);
      }

      style = f.substr(l+2,f.rfind(';')-l-2);

      i=0;
      while((i=style.find(';',i))!=string::npos) {
         style.erase(i, 1);
      }
   }

   /* family name can be optionally terminated by a comma, let's use it to
      allow family names containing spaces. */
   return family_name + ", " + style + " " + size;
}

void VTETerm::SetFont (const string &f)
{
   string font = WxFontStringToPango (f);

   vte_terminal_set_font_from_string ((VteTerminal*)m_widget, font.c_str());
}

int VTETerm::GetCharWidth()
{
   return vte_terminal_get_char_width ((VteTerminal*)m_widget);
}

int VTETerm::GetCharHeight()
{
   return vte_terminal_get_char_height  ((VteTerminal*)m_widget);
}
